<?php

class ID_Recaptcha {
	
	/**
	 * Constructor for ID_Recaptcha class.
	 * 
	 * Initializes the set_filters method.
	 */
	function __construct() {
		self::set_filters();
	}

	/**
	 * Set filters for reCAPTCHA functionality.
	 *
	 * This function sets the filters for reCAPTCHA functionality by adding an action
	 * to the 'plugins_loaded' hook, which calls the recaptcha_load method.
	 */
	function set_filters() {
		add_action('plugins_loaded', array($this, 'recaptcha_load'));
	}

	/**
	 * Load reCAPTCHA functionality.
	 *
	 * This function loads reCAPTCHA functionality if IDC and IDCF are present. It
	 * adds various actions and filters for reCAPTCHA on login, registration, and
	 * other forms. It also verifies reCAPTCHA on login and registration.
	 */
	function recaptcha_load() {
		if (idf_has_idc() && idf_has_idcf()) {
			if (is_id_pro() || is_idc_licensed()) {
				add_action('init', array($this, 'recaptcha_init'));
				add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'));
				add_action('login_enqueue_scripts', array($this, 'enqueue_scripts'));
				// check for idc
				add_action('md_register_extrafields', array($this, 'render_reg_captcha'));
				add_filter('login_form_middle', array($this, 'render_login_captcha'));
				// default forms
				add_action('login_form', array($this, 'echo_login_captcha'));
				// wc forms
				// reserved
				add_action('admin_menu', array($this, 'admin_menus'), 20);
				// Verify on login
				add_filter('authenticate', array($this, 'login_verify_gcaptcha3'), 9, 3);
				// Verify on registration
				add_filter( 'pre_user_login' , array($this, 'register_verify_gcaptcha3') );
			}
		}
	}

	/**
	 * Initialize reCAPTCHA functionality.
	 *
	 * This function initializes reCAPTCHA functionality by registering the necessary scripts.
	 * It is called on the 'init' hook and calls the register_scripts method.
	 */
	function recaptcha_init() {
		self::register_scripts();
	}

	/**
	 * Add admin menus for reCAPTCHA settings.
	 *
	 * This function adds a submenu page under 'idf' for managing reCAPTCHA settings.
	 *
	 * @return void
	 */
	function admin_menus() {
		add_submenu_page('idf', __('reCAPTCHA', 'idf'), __('reCAPTCHA', 'idf'), 'manage_options', 'idc_recaptcha', array($this, 'admin_menu'));
	}

	/**
	 * Admin menu for managing reCAPTCHA settings.
	 *
	 * This function handles the submission of reCAPTCHA settings and includes the
	 * settings menu template for display.
	 *
	 * @return void
	 */
	function admin_menu() {
		$settings = get_option('id_recaptcha_settings');
		if (isset($_POST['submit_id_recaptcha_settings'])) {
			foreach ($_POST as $k=>$v) {
				$settings[$k] = sanitize_text_field($v);
				update_option('id_recaptcha_settings', $settings);
			}
		}
		include_once('templates/admin/_settingsMenu.php');
	}

	/**
	 * Register scripts for reCAPTCHA functionality.
	 *
	 * This function registers the necessary scripts for reCAPTCHA functionality based on the
	 * settings provided. It registers the reCAPTCHA script, the id_recaptcha script, and the
	 * id_recaptcha style. It also localizes the id_recaptcha script with the site ID and
	 * reCAPTCHA version.
	 *
	 * @global string $idf_current_version The current version of the plugin.
	 * @return void
	 */
	function register_scripts() {
		global $idf_current_version;
		$language = get_bloginfo('language');
		$settings = get_option('id_recaptcha_settings');
		if(isset($settings['id_recaptcha_type'])) {
			if($settings['id_recaptcha_type'] == 'v3') {
				wp_register_script('recaptcha', 'https://www.google.com/recaptcha/api.js?render='.$settings['id_recaptcha_site_id'].'&hl='.$language.' async defer');
			} else {
				wp_register_script('recaptcha', 'https://www.google.com/recaptcha/api.js?onload=idRecaptchaLoad&render=explicit&hl='.$language.' async defer');
			}
			wp_register_script('id_recaptcha', plugins_url('js/id_recaptcha-min.js', __FILE__), array(), time());
			wp_register_style('id_recaptcha', plugins_url('css/id_recaptcha-min.css', __FILE__));
			wp_localize_script('id_recaptcha', 'id_recaptcha_site_id', (isset($settings['id_recaptcha_site_id']) ? $settings['id_recaptcha_site_id'] : ''));
			wp_localize_script('id_recaptcha', 'id_recaptcha_version', (isset($settings['id_recaptcha_type']) ? $settings['id_recaptcha_type'] : 'v2'));
		}
	}

	/**
	 * Enqueue scripts for reCAPTCHA functionality.
	 *
	 * This function enqueues the necessary scripts for reCAPTCHA functionality based on the
	 * settings provided. It enqueues the jQuery script, the reCAPTCHA script, the id_recaptcha
	 * script, and the id_recaptcha style if the site ID is available.
	 *
	 * @return void
	 */
	function enqueue_scripts() {
		if ($this::has_site_id()) {
			wp_enqueue_script('jquery');
			wp_enqueue_script('recaptcha');
			wp_enqueue_script('id_recaptcha');
			wp_enqueue_style('id_recaptcha');
		}
	}

	/**
	 * Check if the site has a reCAPTCHA site ID set.
	 *
	 * This function checks if the site has a reCAPTCHA site ID set in the plugin settings.
	 *
	 * @return bool True if the site has a reCAPTCHA site ID set, false otherwise.
	 */
	function has_site_id() {
		$settings = get_option('id_recaptcha_settings');
		return !empty($settings['id_recaptcha_site_id']);
	}

	/**
	 * Generate the reCAPTCHA content for a form.
	 *
	 * This function generates the reCAPTCHA content for a form based on the plugin settings.
	 *
	 * @param string $wrapper The HTML tag to wrap the reCAPTCHA content in. Default is 'div'.
	 * @return string The generated reCAPTCHA content.
	 */
	function captcha_content($wrapper = 'div') {
		$settings = get_option('id_recaptcha_settings');
		if(isset($settings['id_recaptcha_type']) && $settings['id_recaptcha_type'] == 'v3') {
			return '';
		} else {
			return '<'.$wrapper.' class="form-row id_recaptcha_placeholder" data-callback="idRecaptchaCallback"></'.$wrapper.'>';
		}
	}

	/**
	 * Render the reCAPTCHA content for registration form.
	 *
	 * This function renders the reCAPTCHA content for the registration form.
	 */
	function render_reg_captcha() {
		echo $this::captcha_content();
	}

	/**
	 * Render the reCAPTCHA content for login form.
	 *
	 * This function renders the reCAPTCHA content for the login form.
	 *
	 * @param string $content The content to be rendered. Default is an empty string.
	 * @return string The rendered reCAPTCHA content.
	 */
	function render_login_captcha($content = '') {
		return self::captcha_content('p');
	}

	/**
	 * Echo the reCAPTCHA content for login form.
	 *
	 * This function echoes the reCAPTCHA content for the login form if the site has a reCAPTCHA site ID set.
	 */
	function echo_login_captcha() {
		if ($this::has_site_id()) {
			echo self::render_login_captcha();
		}
	}

	/**
	 * Verify reCAPTCHA on login.
	 *
	 * This function checks the reCAPTCHA response on login and returns a WP_Error
	 * if the reCAPTCHA is not verified or not submitted. It uses the secret key
	 * to verify the response with Google's reCAPTCHA API.
	 *
	 * @return WP_Error|void Returns WP_Error if reCAPTCHA verification fails.
	 */
	function login_verify_gcaptcha3() {
		if( isset($_POST['g-recaptcha-response']) ) {
			$settings = get_option('id_recaptcha_settings');
			$secret   = $settings['id_recaptcha_secret_key'];
			$captcha = $_POST['g-recaptcha-response'];
			$action = "login";
			// call curl to POST request
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL,"https://www.google.com/recaptcha/api/siteverify");
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query(
				array(
					'secret' => $secret, 
					'response' => $captcha
				)
			));
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			$response = curl_exec($ch);
			curl_close($ch);
			$arrResponse = json_decode($response, true);
			// verify the response
			if($arrResponse["success"] == '1' && $arrResponse["action"] == $action && $arrResponse["score"] >= 0.5) {
				// valid submission
			} else {
				// spam submission
				$error = new WP_Error();
				$user  = new WP_Error( 'authentication_failed', __( 'ERROR: Recaptcha not verified.' ) );
				return $error;	
			}
		} else {
			$error = new WP_Error();
			$user  = new WP_Error( 'authentication_failed', __( 'ERROR: Recaptcha is not submitted.' ) );
			return $error;
		}
	}

	/**
	 * Register and verify reCAPTCHA for user registration.
	 *
	 * This function processes the reCAPTCHA response submitted with the user
	 * registration form. It sends a request to Google's reCAPTCHA API for
	 * verification and handles the response. If the reCAPTCHA is verified,
	 * the function returns true, otherwise it outputs a JSON encoded failure
	 * response and exits.
	 *
	 * @return bool True if reCAPTCHA is verified, otherwise exits script.
	 */
	function register_verify_gcaptcha3() {
		if( isset($_POST['Fields']) ) {
			foreach($_POST['Fields'] as $f) {
				if($f['name']=='g-recaptcha-response' && !empty($f['value'])) {
					$settings = get_option('id_recaptcha_settings');
					$secret   = $settings['id_recaptcha_secret_key'];
					$captcha = $f['value'];
					$action = "register";
					// call curl to POST request
					$ch = curl_init();
					curl_setopt($ch, CURLOPT_URL,"https://www.google.com/recaptcha/api/siteverify");
					curl_setopt($ch, CURLOPT_POST, 1);
					curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query(
						array(
							'secret' => $secret, 
							'response' => $captcha
						)
					));
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
					$response = curl_exec($ch);
					curl_close($ch);
					$arrResponse = json_decode($response, true);
					// verify the response
					if($arrResponse["success"] == '1') {
						// valid submission
						
					} else {
						// spam submission
						print_r(json_encode(array('response' => 'failure', 'message' => __( 'ERROR: Recaptcha not verified.' ))));
						exit;	
					}
					break;
				} else {
					print_r(json_encode(array('response' => 'failure', 'message' => __( 'ERROR: Recaptcha is not submitted.' ))));
					exit;	
				}
			}
		} else {
			print_r(json_encode(array('response' => 'failure', 'message' => __( 'ERROR: Recaptcha is not submitted..' ))));
			exit;	
		}
		return true;
	}
}
new ID_Recaptcha(); ?>